﻿#include "A7100AnalyzerSettings.h"
#include <AnalyzerHelpers.h>
#include <sstream>
#include <cstring>
#include <chrono>
#include <thread>
#include <X11/Xlib.h>
#include <X11/keysym.h>
#include <X11/extensions/XTest.h>

#pragma warning(disable: 4800) //warning C4800: 'U32' : forcing value to bool 'true' or 'false' (performance warning)
#define CHANNEL_SYN_NAME "BSYN"
#define CHANNEL_VID1_NAME "VID1"
#define CHANNEL_VID2_NAME "VID2"


A7100AnalyzerSettings::A7100AnalyzerSettings()
    :   mInputChannel1(UNDEFINED_CHANNEL),
        mInputChannel2(UNDEFINED_CHANNEL),
        mInputChannel3(UNDEFINED_CHANNEL)
{
    mInputChannelInterface1.reset(new AnalyzerSettingInterfaceChannel());
    mInputChannelInterface1->SetTitleAndTooltip(CHANNEL_SYN_NAME, "BSYN Sync Signal");
    mInputChannelInterface1->SetChannel(mInputChannel1);

    mInputChannelInterface2.reset(new AnalyzerSettingInterfaceChannel());
    mInputChannelInterface2->SetTitleAndTooltip(CHANNEL_VID1_NAME, "Video 1");
    mInputChannelInterface2->SetChannel(mInputChannel2);

    mInputChannelInterface3.reset(new AnalyzerSettingInterfaceChannel());
    mInputChannelInterface3->SetTitleAndTooltip(CHANNEL_VID2_NAME, "Video 2");
    mInputChannelInterface3->SetChannel(mInputChannel3);

    mShowWindow.reset(new AnalyzerSettingInterfaceBool());
    mShowWindow->SetTitleAndTooltip("", "Zeige das Bildschirm-Fenster");
    mShowWindow->SetCheckBoxText("Zeige Fenster");
    mShowWindow->SetValue(screenVisible);

    AddInterface(mInputChannelInterface1.get());
    AddInterface(mInputChannelInterface2.get());
    AddInterface(mInputChannelInterface3.get());
    AddInterface(mShowWindow.get());

    ClearChannels();
    AddChannel(mInputChannel1, CHANNEL_SYN_NAME, false);
    AddChannel(mInputChannel2, CHANNEL_VID1_NAME, false);
    AddChannel(mInputChannel3, CHANNEL_VID2_NAME, false);
}

A7100AnalyzerSettings::~A7100AnalyzerSettings()
{
}

bool A7100AnalyzerSettings::SetSettingsFromInterfaces()
{
    mInputChannel1 = mInputChannelInterface1->GetChannel();
    mInputChannel2 = mInputChannelInterface2->GetChannel();
    mInputChannel3 = mInputChannelInterface3->GetChannel();

    ClearChannels();
    AddChannel(mInputChannel1, CHANNEL_SYN_NAME, true);
    AddChannel(mInputChannel2, CHANNEL_VID1_NAME, true);
    AddChannel(mInputChannel3, CHANNEL_VID2_NAME, true);

    if (screenVisible && !mShowWindow->GetValue())
    {
        screenVisible = false;
    }

    if (!screenVisible && mShowWindow->GetValue())
    {
        if (winThread.joinable()) winThread.join();
        winThread = move(std::thread(&A7100AnalyzerSettings::WindowThread, this));
    }
    return true;
}

void A7100AnalyzerSettings::UpdateInterfacesFromSettings()
{
    mInputChannelInterface1->SetChannel(mInputChannel1);
    mInputChannelInterface2->SetChannel(mInputChannel2);
    mInputChannelInterface3->SetChannel(mInputChannel3);
    mShowWindow->SetValue(screenVisible);
}

void A7100AnalyzerSettings::LoadSettings(const char *settings)
{
    SimpleArchive text_archive;
    text_archive.SetString(settings);

    const char *name_string;    //the first thing in the archive is the name of the protocol analyzer that the data belongs to.
    text_archive >> &name_string;
    if (strcmp(name_string, "A7100Analyzer") != 0) {
        AnalyzerHelpers::Assert("A7100Analyzer: Provided with a settings string that doesn't belong to us;");
    }

    text_archive >> mInputChannel1;
    text_archive >> mInputChannel2;
    text_archive >> mInputChannel3;

    ClearChannels();
    AddChannel(mInputChannel1, CHANNEL_SYN_NAME, true);
    AddChannel(mInputChannel2, CHANNEL_VID1_NAME, true);
    AddChannel(mInputChannel3, CHANNEL_VID2_NAME, true);

    UpdateInterfacesFromSettings();
}

const char *A7100AnalyzerSettings::SaveSettings()
{
    SimpleArchive text_archive;

    text_archive << "A7100Analyzer";
    text_archive << mInputChannel1;
    text_archive << mInputChannel2;
    text_archive << mInputChannel3;

    return SetReturnString(text_archive.GetString());
}

Window window_from_name_search(Display *display, Window current, char const *needle)
{
    Window retval, root, parent, *children;
    unsigned children_count;
    char *name = NULL;

    /* Check if this window has the name we seek */
    if(XFetchName(display, current, &name) > 0) {
        int r = strcmp(needle, name);
        XFree(name);
        if(r == 0) {
            return current;
        }
    }

    retval = 0;

    /* If it does not: check all subwindows recursively. */
    if(0 != XQueryTree(display, current, &root, &parent, &children, &children_count)) {
        unsigned i;
        for(i = 0; i < children_count; ++i) {
            Window win = window_from_name_search(display, children[i], needle);

            if(win != 0) {
                    retval = win;
                break;
            }
        }
        XFree(children);
    }
    return retval;
}

XKeyEvent createKeyEvent(Display *display, Window &win, Window &winRoot, bool press, int keycode, int modifiers)
{
	XKeyEvent event;

	event.display     = display;
	event.window      = win;
	event.root        = winRoot;
	event.subwindow   = None;
	event.time        = CurrentTime;
	event.x           = 1;
	event.y           = 1;
	event.x_root      = 1;
	event.y_root      = 1;
	event.same_screen = True;
	event.keycode     = XKeysymToKeycode(display, keycode);
	event.state       = modifiers;
	if(press)
		event.type = KeyPress;
	else
		event.type = KeyRelease;

	return event;
}


void A7100AnalyzerSettings::WindowThread()
{
    fprintf(stderr, "Start window thread\n");

    Display *display;
    Window window;
    XEvent event;
    int screen;
    Window mainwin;

    display = XOpenDisplay(NULL);
    if (display == NULL)
    {
            fprintf(stderr, "Cannot open display\n");
            return;
    }
    screen = DefaultScreen(display);

    mainwin = window_from_name_search(display, RootWindow(display, screen), "KingstVIS");

    window = XCreateSimpleWindow(display, RootWindow(display, screen), 100, 100, screenWidth, screenHeight, 1, BlackPixel(display, screen), WhitePixel(display, screen));

    XStoreName(display, window, "A7100 Analyzer Screen Capture (Auto-F5: OFF)");

    Atom del_window = XInternAtom(display, "WM_DELETE_WINDOW", 0);
    XSetWMProtocols(display, window, &del_window, 1);
    XMapWindow(display, window);
    XSelectInput(display, window, ButtonPressMask|KeyPressMask);

    screenVisible = true;
    mShowWindow->SetValue(screenVisible);

    while (event.type != ClientMessage && screenVisible)
    {
        if (XPending(display)>0)
        {
            XNextEvent(display, &event);
            if (event.type==ButtonPress)
            {
                autoRefresh = !autoRefresh;
                if (autoRefresh)
                {
                    XStoreName(display, window, "A7100 Analyzer Screen Capture (Auto-F5: ON)");
                }
                else
                {
                    XStoreName(display, window, "A7100 Analyzer Screen Capture (Auto-F5: OFF)");
                }
            }
        }
        if (calcDone)
        {
            XImage* img = XCreateImage(display, DefaultVisual(display, 0), 24, ZPixmap, 0, (char*)screenMap, screenWidth, screenHeight, 32, 0);
            XPutImage(display, window, XCreateGC(display, window, 0, NULL), img, 0, 0, 0, 0, screenWidth, screenHeight);
            if (autoRefresh)
            {
                unsigned int keycode;
                keycode = XKeysymToKeycode(display, XK_F5);
                XTestFakeKeyEvent(display, keycode, True, 0);
                XTestFakeKeyEvent(display, keycode, False, 0);
            }
            calcDone = false;
            XFlush(display);
        }
        std::this_thread::sleep_for(std::chrono::milliseconds(10));
    }

    screenVisible = false;
    mShowWindow->SetValue(screenVisible);
    XDestroyWindow(display, window);
    XCloseDisplay(display);

    fprintf(stderr, "End window thread\n");
}
